/*
 * Simple NIS map accessibility checker. Prints the first record when it
 * succeeds.
 *
 * Author: Wietse Venema.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <rpc/rpc.h>
#include <netdb.h>
#include <tcp.h>

#include "yp.h"

static struct timeval timeout = { 5, 0 };
static int    verbose = 0;
static char  *progname;

#define debug if (verbose) printf

static void usage (void)
{
  fprintf (stderr, "usage: %s [-dv] [-t timeout] domain map server\n",
           progname);
  exit (0);
}

/*
 * make_tcp_client - create client handle to talk to rpc server
 */

static CLIENT *make_tcp_client (struct sockaddr_in *addr,
                                u_long program, u_long version)
{
  static tcp_Socket sock;

  debug ("Trying to set up TCP client handle\n");
  addr->sin_port = 0;
  memset (&sock,0,sizeof(sock));
  return clnttcp_create (addr, program, version, (int*)&sock, 0,0);
}

/*
 * make_udp_client - create client handle to talk to rpc server
 */
static CLIENT *make_udp_client (struct sockaddr_in *addr,
                                u_long program, u_long version)
{
  static udp_Socket sock;

  debug ("Trying to set up UDP client handle\n");
  addr->sin_port = 0;
  memset (&sock,0,sizeof(sock));
  return clntudp_create (addr, program, version, timeout, (int*)&sock);
}

/*
 * find_host - look up host information
 */
static int find_host (struct sockaddr_in *sin, char *host)
{
  struct hostent *hp;

  /* Look up IP address information. XXX with multi-homed hosts, should try
   * all addresses until we succeed.
   */
  memset (sin, 0, sizeof(*sin));
  sin->sin_family = AF_INET;

  debug ("Looking up host %s\n", host);
  sin->sin_addr.s_addr = inet_addr (host);
  if (sin->sin_addr.s_addr != INADDR_NONE)
     return (1);

  hp = gethostbyname (host);
  if (!hp || hp->h_addrtype != AF_INET)
     return (0);

  memcpy (&sin->sin_addr, hp->h_addr, sizeof(sin->sin_addr));
  return (1);
}

/*
 * try_map - transfer first map entry
 */
static int try_map (CLIENT *client, char *domain, char *map)
{
  struct YPreq_nokey     nokey;
  struct YPresp_key_val  key_val;
  enum   clnt_stat       status;
  char   keybuf [1024];
  char   valbuf [1024];

  nokey.domain = domain;
  nokey.map    = map;

  key_val.key.keydat_val = keybuf;
  key_val.key.keydat_len = sizeof(keybuf);
  key_val.val.valdat_val = valbuf;
  key_val.val.valdat_len = sizeof(valbuf);

  /* Look up the first entry. Sending the call may fail.
   */
  debug ("Trying: %s %s\n", domain, map);
  status = clnt_call (client, YPPROC_FIRST, xdr_YPreq_nokey,
                      (char*)&nokey, xdr_YPresp_key_val,
                      (char*)&key_val, timeout);
  if (status != RPC_SUCCESS)
  {
    clnt_perrno (status);
    return (0);
  }

  /* The call itself may fail (wrong domain or map).
   */
  if (key_val.stat != YP_TRUE)
  {
    fprintf (stderr, "%s: domain %s map %s: failed\n",
             progname, domain, map);
    exit (1);
  }

  /* Show just one entry as proof.
   */
  printf ("%.*s\n", (int)key_val.key.keydat_len, key_val.key.keydat_val);
  return (1);
}

int main (int argc, char **argv)
{
  struct sockaddr_in sin;
  CLIENT *client;
  char   *domain;
  char   *map;
  char   *host;
  int     success = 0;
  int     opt;

  progname = argv[0];

  /* Parse comand-line
   */
  while ((opt = getopt(argc, argv, "?dvt:")) != EOF)
  {
    switch (opt)
    {
      case 'd':
           dbug_init();
           break;
      case 'v':                               /* turn on verbose mode */
           verbose = 1;
           break;
      case 't':                               /* change timeout */
           timeout.tv_sec = atoi (optarg);
           break;
      case '?':
      default:
           usage();
    }
  }
  argc -= optind;
  argv += optind;

  if (argc < 3)
     usage();

  domain = *argv++;;
  map    = *argv++;
  host   = *argv++;

  rpc_init();
  sock_init();

  if (!find_host(&sin, host))
  {
    fprintf (stderr, "%s: unknown host: %s\n", progname, host);
    return (1);
  }

  /* Now try each transport until we succeed.
   */
  if ((client = make_tcp_client(&sin, YPPROG, YPVERS)) == 0 ||
      (success = try_map(client, domain, map)) == 0)
  {
    if ((client = make_udp_client(&sin, YPPROG, YPVERS)) != 0)
       success = try_map (client, domain, map);
  }
  return (success ? 0 : -1);
}
